<?php

trait PropertySettingEntityTemplateComponentTrait {

  /**
   * {@inheritdoc}
   */
  public function suggestConfigKey(): string {
    return $this->definition['property'] ?: $this->definition['item'];
  }

  /**
   * {@inheritdoc}
   */
  public function label() : string {
    $structure = $this->templateConfig->getStructure();

    if (
      $structure instanceof EntityStructureWrapper &&
      ($info = $structure->getPropertyInfo($this->definition['property']))
    ) {
      return $info['label'] ?? $this->definition['property'];
    }
    else if (
      $structure instanceof EntityStructureWrapper &&
      $this->definition['property'] === '__self'
    ) {
      return $structure->info()['label'] ?? 'Self';
    }
    else if ($structure instanceof EntityListWrapper) {
      return t('Item');
    }

    return t(
      'Invalid of missing property "@property".',
      array('@property' => $this->definition['property'] ?? 'unknown')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function description() : string {
    $structure = $this->templateConfig->getStructure();

    if (
      $structure instanceof EntityStructureWrapper &&
      ($info = $structure->getPropertyInfo($this->definition['property']))
    ) {
      return t('@method value for @property', array(
        '@method' => $this->inputMethodLabel(),
        '@property' => $info['label'] ?? $this->definition['property'],
      ));
    }
    else if (
      $structure instanceof EntityStructureWrapper &&
      $this->definition['property'] === '__self'
    ) {
      return t('@method value for @property', array(
        '@method' => $this->inputMethodLabel(),
        '@property' => $info['label'] ?? $this->definition['property'],
      ));
    }
    else if ($structure instanceof EntityListWrapper) {
      return t('@method new item to @property', array(
        '@method' => $this->inputMethodLabel(),
        '@property' => $structure->info()['label'] ?? 'Self',
      ));
    }

    return t(
      'Invalid of missing property "@property".',
      array('@property' => $this->definition['property'] ?? 'unknown')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function needsSetup() : bool {
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function setupForm($form, &$form_state) {
    /** @var \EntityStructureWrapper $structure */
    $structure = $this->templateConfig->getStructure();

    if (!isset($form_state['definition'])) {
      $form_state['definition'] = array();
    }

    if ($structure instanceof EntityStructureWrapper) {
      $form['property'] = [
        '#type' => 'select',
        '#title' => t('Property'),
        '#required' => TRUE,
        '#options' => $this->applicableProperties($structure),
        '#default_value' => $form_state['definition']['property'] ?? ($this->definition['property'] ?? NULL),
      ];
    }
    else if ($structure instanceof EntityListWrapper) {
      $form['item'] = [
        '#type' => 'select',
        '#title' => t('Item'),
        '#required' => TRUE,
        '#options' => [
          '__new_append' => t('New Item (Append)'),
        ],
        '#default_value' => $form_state['definition']['item'] ?? ($this->definition['item'] ?? '__new_append'),
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function setupFormValidate($form, &$form_state) {
    // Do nothing.
  }

  /**
   * {@inheritdoc}
   */
  public function setupFormSubmit($form, &$form_state) {
    $values = drupal_array_get_nested_value($form_state['values'], $form['#parents']);
    if (isset($values['property'])) {
      $this->definition['property'] = $values['property'];
    }
    if (isset($values['item'])) {
      $this->definition['item'] = $values['item'];
    }
  }


  /**
   * Get the target property definition.
   *
   * @return array
   */
  public function getTargetPropertyDefinition() : array {
    $structure = $this->templateConfig->getStructure();
    return $structure instanceof EntityListWrapper ?
      $structure[$structure->count()]->info() :
      (
        $this->definition['property'] === '__self' ?
          $structure->info() :
          $structure->getPropertyInfo($this->definition['property'])
      );
  }

  public function getTargetPropertyWrapper(EntityMetadataWrapper $structure) : EntityMetadataWrapper {
    if ($structure instanceof EntityListWrapper) {
      return $structure[$structure->count()];
    }
    else if ($structure instanceof EntityStructureWrapper) {
      return $this->definition['property'] === '__self' ?
        $structure :
        $structure->get($this->definition['property']);
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function applies(TemplateConfiguration $template_configuration): bool {
    return $template_configuration->getStructure() instanceof EntityStructureWrapper ||
      $template_configuration->getStructure() instanceof EntityListWrapper;
  }

  /**
   * Get the properties this can be applied to.
   *
   * @param \EntityStructureWrapper $structure
   *   The structure.
   *
   * @return array
   */
  protected function applicableProperties(EntityStructureWrapper $structure) {
    $props = [
      '__self' => t('Self'),
    ];

    foreach ($structure->getPropertyInfo() as $name => $info) {
      if (!empty($info['computed']) || empty($info['setter callback'])) {
        continue;
      }

      $props[$name] = $info['label'];
    }

    return $props;
  }

  /**
   * Can this swap for the given component.
   *
   * @param \EntityTemplateComponentInterface $component
   *   The component to be swapped out.
   *
   * @return bool
   */
  public function canSwapFor(EntityTemplateComponentInterface $component) : bool {
    if (
      $component instanceof PropertySettingEntityComponentInterface
      && !empty($component->getDefinition()['property'])
    ) {
      $app_properties = $this->applicableProperties($this->templateConfig->getStructure());
      return isset($app_properties[$component->getDefinition()['property']]);
    }

    return FALSE;
  }

  /**
   * Prepare the swap definition for this component.
   *
   * @param \EntityTemplateComponentInterface $component
   *   The component being replaced
   *
   * @return array
   */
  public function prepareSwapDefinitionFor(EntityTemplateComponentInterface $component) : array {
    return array_filter([
      'property' => $component->getDefinition()['property'],
    ]);
  }

  /**
   * Prepare the swap configuration for this component.
   *
   * @param \EntityTemplateComponentInterface $component
   *   The component being swapped.
   *
   * @return array
   */
  public function prepareSwapConfigurationFor(EntityTemplateComponentInterface $component) : array {
    return [];
  }

  /**
   * Get the input method label.
   *
   * @return string
   */
  abstract protected function inputMethodLabel() : string;


}
